# Zenoss JSON API Helper
#
#
# To quickly explore, execute 'python -i api_helper.py'
#
# >>> z = ZenossAPIHelper()
# >>> events = z.get_events()
# etc.


import json
import urllib
import urllib2

ZENOSS_INSTANCE = 'http://localhost:8080'
ZENOSS_USERNAME = 'admin'
ZENOSS_PASSWORD = 'zenoss'

ROUTERS = { 'MessagingRouter': 'messaging',
            'EventsRouter': 'evconsole',
            'ProcessRouter': 'process',
            'ServiceRouter': 'service',
            'DeviceRouter': 'device',
            'NetworkRouter': 'network',
            'TemplateRouter': 'template',
            'DetailNavRouter': 'detailnav',
            'ReportRouter': 'report',
            'MibRouter': 'mib',
            'ZenPackRouter': 'zenpack',
            'ImpactRouter': 'enterpriseservices' }

IMPACT_ROOT = '/zport/dmd/DynamicServices'

class ZenossAPIHelper():
    def __init__(self, debug=False):
        """
        Initialize the API connection, log in, and store authentication cookie
        """
        # Use the HTTPCookieProcessor as urllib2 does not save cookies by default
        self.urlOpener = urllib2.build_opener(urllib2.HTTPCookieProcessor())
        if debug: self.urlOpener.add_handler(urllib2.HTTPHandler(debuglevel=1))
        self.reqCount = 1

        # Contruct POST params and submit login.
        loginParams = urllib.urlencode(dict(
                        __ac_name = ZENOSS_USERNAME,
                        __ac_password = ZENOSS_PASSWORD,
                        submitted = 'true',
                        came_from = ZENOSS_INSTANCE + '/zport/dmd'))
        self.urlOpener.open(ZENOSS_INSTANCE + '/zport/acl_users/cookieAuthHelper/login',
                            loginParams)

    def _router_request(self, router, method, data=[]):
        if router not in ROUTERS:
            raise Exception('Router "' + router + '" not available.')

        # Contruct a standard URL request for API calls
        req = urllib2.Request(ZENOSS_INSTANCE + '/zport/dmd/' +
                              ROUTERS[router] + '_router')

        # NOTE: Content-type MUST be set to 'application/json' for these requests
        req.add_header('Content-type', 'application/json; charset=utf-8')

        # Convert the request parameters into JSON
        reqData = json.dumps([dict(
                    action=router,
                    method=method,
                    data=data,
                    type='rpc',
                    tid=self.reqCount)])

        # Increment the request count ('tid'). More important if sending multiple
        # calls in a single request
        self.reqCount += 1

        # Submit the request and convert the returned JSON to objects
        return json.loads(self.urlOpener.open(req, reqData).read())

    ###
    ### Non-impact API calls
    ###

    def get_devices(self, deviceClass='/zport/dmd/Devices'):
        return self._router_request('DeviceRouter', 'getDevices',
                                    data=[{'uid': deviceClass,
                                           'params': {} }])['result']

    def get_events(self, device=None, component=None, eventClass=None):
        data = dict(start=0, limit=100, dir='DESC', sort='severity')
        data['params'] = dict(severity=[5,4,3,2], eventState=[0,1])

        if device: data['params']['device'] = device
        if component: data['params']['component'] = component
        if eventClass: data['params']['eventClass'] = eventClass

        return self._router_request('EventsRouter', 'query', [data])['result']

    def add_device(self, deviceName, deviceClass):
        data = dict(deviceName=deviceName, deviceClass=deviceClass)
        return self._router_request('DeviceRouter', 'addDevice', [data])

    def create_event_on_device(self, device, severity, summary):
        if severity not in ('Critical', 'Error', 'Warning', 'Info', 'Debug', 'Clear'):
            raise Exception('Severity "' + severity +'" is not valid.')

        data = dict(device=device, summary=summary, severity=severity,
                    component='', evclasskey='', evclass='')
        return self._router_request('EventsRouter', 'add_event', [data])

    ###
    ### Impact API calls
    ###

    def impact_request(self, method, data=[]):
        return self._router_request('ImpactRouter', method, data=data)['result']

    def get_services_tree(self):
        return self.impact_request('getTree')[0]

    def get_service_info(self, uid):
        data = {'uid': uid}
        return self.impact_request('getInfo', [data])

    def delete_node(self, uid):
        data = {'uid': uid}
        return self.impact_request('deleteNode', [data])

    def create_organizer(self, contextUid, id):
        data = dict(contextUid=contextUid, id=id)
        return self.impact_request('addOrganizer', [data])

    def create_service(self, contextUid, id):
        data = dict(contextUid=contextUid, id=id)
        return self.impact_request('addNode', [data])

    def add_impact(self, parentUid, childUids):
        if isinstance(childUids, basestring):
            childUids = [childUids]
        data = dict(targetUid=parentUid, uids=childUids)
        return self.impact_request('addToDynamicService', [data])
